#include "ell/Basin.hh"
#include "ell/Walk.hh"
#include "ell/SC_Outstream.hh"
#include "ell/SC_Counting.hh"
#include "ell/spinglass/S_SG_Ising.hh"
#include "biu/RandomNumberGenerator.hh"
#include "biu/RandomNumberFactory.hh"
#include <iostream>
#include <iomanip>


int main( int argc, char** argv ) {

	using namespace ell;

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test function : basin\n"
				<<"===========================================" <<std::endl;

		// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	const unsigned int seed = 1;
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	
		// start tests
	size_t l = 8;
	std::cout <<"->new S_SG_Ising("<<l<<",random) = "; std::cout.flush();
	S_SG_Ising* sgi = new S_SG_Ising(l,true);
	std::cout <<sgi->toString() <<std::endl;
	SC_Counting sccAccepted;
	SC_Counting sccChecked;
	
	std::cout <<"->gradientWalk --> lm = "; std::cout.flush();
	WalkGradient::walkGradient(sgi, &sccAccepted);
	std::cout <<sccAccepted.getLastAdded()->toString() <<std::endl;
	
	SC_Outstream scoAccepted(std::cout, " <-- basin");
	SC_Outstream scoChecked(std::cout, "     not");
	
	// check basin algorithm implementation
	
	std::cout <<"->basin( lm )" <<std::endl;
	double maxE = Basin::basin(	sccAccepted.getLastAdded(),
								&scoAccepted,
								99999.0,
								1.0,
								UINT_MAX,
								&scoChecked
								);
	std::cout <<" max E flooded = "<<std::fixed <<std::setprecision(3) <<maxE <<std::endl;

	// check Wrapper class
	
	std::cout <<"->Basin bas()" <<std::endl;
	Basin bas(99999.0, 1.0, UINT_MAX);
	std::cout <<"->bas.flood( lm )" <<std::endl;
	maxE = bas.flood(sccAccepted.getLastAdded(), &scoAccepted, &scoChecked);
	std::cout <<" max E flooded = "<<std::fixed <<std::setprecision(3) <<maxE <<std::endl;
	
	delete sgi;

  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;

	return 0;	
}
